;~ #include-once

;http://shapelib.maptools.org/dbf_api.html
;http://sourceforge.net/projects/dbase/

Global Enum $DBF_FTString, $DBF_FTInteger, $DBF_FTDouble, $DBF_FTLogical, $DBF_FTInvalid
Global Const $DBF_StringNULL = "NULL"
Global Const $DBF_StringTRUE = "True"
Global Const $DBF_StringFALSE = "False"

Global $sDLL_DBF = @ScriptDir & "\DBF.dll"
Global $hDLL_DBF = 0

Func _DBF_Init($sDLL = $sDLL_DBF)
	If $hDLL_DBF > 0 Then Return 1
	$hDLL_DBF = DllOpen($sDLL)
	If $hDLL_DBF = -1 Then
		Return SetError(1, 0, 0)
	EndIf
	OnAutoItExitRegister("_DBF_DeInit")
	Return 1
EndFunc   ;==>_DBF_Init

Func _DBF_DeInit()
	DllClose($hDLL_DBF)
EndFunc   ;==>_DBF_DeInit

Func _DBF_Open($sFileName, $sMode = "rb")
	;	"rb" (read-only binary) and "rb+" (read/write binary)
	If _DBF_Init() = 0 Then Return SetError(1, 0, 0)
	Local $aRet = DllCall($hDLL_DBF, "handle:cdecl", "DBFOpen", "str", $sFileName, "str", $sMode)
	Return $aRet[0]
EndFunc   ;==>_DBF_Open

Func _DBF_Create($sFileName)
	If _DBF_Init() = 0 Then Return SetError(1, 0, 0)
	Local $aRet = DllCall($hDLL_DBF, "handle:cdecl", "DBFCreate", "str", $sFileName)
	Return $aRet[0]
EndFunc   ;==>_DBF_Create

Func _DBF_Close($hDBF)
	Local $aRet = DllCall($hDLL_DBF, "none:cdecl", "DBFClose", "handle", $hDBF)
EndFunc   ;==>_DBF_Close

Func _DBF_GetFieldCount($hDBF)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFGetFieldCount", "handle", $hDBF)
	Return $aRet[0]
EndFunc   ;==>_DBF_GetFieldCount

Func _DBF_GetRecordCount($hDBF)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFGetRecordCount", "handle", $hDBF)
	Return $aRet[0]
EndFunc   ;==>_DBF_GetRecordCount

Func _DBF_GetFieldIndex($hDBF, $sFieldName)
	;0 is first
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFGetFieldIndex", "handle", $hDBF, "str", $sFieldName)
	Return $aRet[0]
EndFunc   ;==>_DBF_GetFieldIndex

Func _DBF_GetFieldInfo($hDBF, $iField)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFGetFieldInfo", "handle", $hDBF, "int", $iField, "str", "", "int*", 0, "int*", 0)
	Local $aRes[4] = [$aRet[0], $aRet[3], $aRet[4], $aRet[5]]
	Return $aRes
EndFunc   ;==>_DBF_GetFieldInfo

Func _DBF_AddField($hDBF, $sFieldName, $iFieldType, $iWidth, $iDecimals)
	;works only if no records in the file
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFAddField", "handle", $hDBF, "str", $sFieldName, "int", $iFieldType, "int", $iWidth, "int", $iDecimals)
	Return $aRet[0]
EndFunc   ;==>_DBF_AddField

Func _DBF_ReadIntegerAttribute($hDBF, $iRecord, $iField)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFReadIntegerAttribute", "handle", $hDBF, "int", $iRecord, "int", $iField)
	Return $aRet[0]
EndFunc   ;==>_DBF_ReadIntegerAttribute

Func _DBF_ReadDoubleAttribute($hDBF, $iRecord, $iField)
	Local $aRet = DllCall($hDLL_DBF, "double:cdecl", "DBFReadDoubleAttribute", "handle", $hDBF, "int", $iRecord, "int", $iField)
	Return $aRet[0]
EndFunc   ;==>_DBF_ReadDoubleAttribute

Func _DBF_ReadStringAttribute($hDBF, $iRecord, $iField)
	Local $aRet = DllCall($hDLL_DBF, "str:cdecl", "DBFReadStringAttribute", "handle", $hDBF, "int", $iRecord, "int", $iField)
	Return $aRet[0]
EndFunc   ;==>_DBF_ReadStringAttribute

Func _DBF_ReadLogicalAttribute($hDBF, $iRecord, $iField)
	Local $aRet = DllCall($hDLL_DBF, "str:cdecl", "DBFReadLogicalAttribute", "handle", $hDBF, "int", $iRecord, "int", $iField)
	Switch $aRet[0]
		Case "T"
			Return $DBF_StringTRUE
		Case "F"
			Return $DBF_StringFALSE
		Case Else
			Return $DBF_StringNULL
	EndSwitch
EndFunc   ;==>_DBF_ReadLogicalAttribute

Func _DBF_IsAttributeNULL($hDBF, $iRecord, $iField)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFIsAttributeNULL", "handle", $hDBF, "int", $iRecord, "int", $iField)
	Return $aRet[0]
EndFunc   ;==>_DBF_IsAttributeNULL

Func _DBF_WriteIntegerAttribute($hDBF, $iRecord, $iField, $nValue)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFWriteIntegerAttribute", "handle", $hDBF, "int", $iRecord, "int", $iField, "int", $nValue)
	Return $aRet[0]
EndFunc   ;==>_DBF_WriteIntegerAttribute

Func _DBF_WriteDoubleAttribute($hDBF, $iRecord, $iField, $nValue)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFWriteDoubleAttribute", "handle", $hDBF, "int", $iRecord, "int", $iField, "double", $nValue)
	Return $aRet[0]
EndFunc   ;==>_DBF_WriteDoubleAttribute

Func _DBF_WriteStringAttribute($hDBF, $iRecord, $iField, $nValue)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFWriteStringAttribute", "handle", $hDBF, "int", $iRecord, "int", $iField, "str", $nValue)
	Return $aRet[0]
EndFunc   ;==>_DBF_WriteStringAttribute

Func _DBF_WriteLogicalAttribute($hDBF, $iRecord, $iField, $nValue)
	Local $cLogical = Asc("F")
	If $nValue Then $cLogical = Asc("T")
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFWriteLogicalAttribute", "handle", $hDBF, "int", $iRecord, "int", $iField, "BYTE", $cLogical)
	Return $aRet[0]
EndFunc   ;==>_DBF_WriteLogicalAttribute

Func _DBF_WriteNULLAttribute($hDBF, $iRecord, $iField)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFWriteNULLAttribute", "handle", $hDBF, "int", $iRecord, "int", $iField)
	Return $aRet[0]
EndFunc   ;==>_DBF_WriteNULLAttribute

Func _DBF_IsRecordDeleted($hDBF, $iRecord)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFIsRecordDeleted", "handle", $hDBF, "int", $iRecord)
	Return $aRet[0]
EndFunc   ;==>_DBF_IsRecordDeleted

Func _DBF_MarkRecordDeleted($hDBF, $iRecord, $iDeleted)
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFMarkRecordDeleted", "handle", $hDBF, "int", $iRecord, "int", $iDeleted)
	Return $aRet[0]
EndFunc   ;==>_DBF_MarkRecordDeleted

Func _DBF_GetNativeFieldType($hDBF, $iField)
	Local $aRet = DllCall($hDLL_DBF, "BYTE:cdecl", "DBFGetNativeFieldType", "handle", $hDBF, "int", $iField)
	Return Chr($aRet[0])
EndFunc   ;==>_DBF_GetNativeFieldType

Func _DBF_DeleteField($hDBF, $iField)
	;works only if no records in the file
	Local $aRet = DllCall($hDLL_DBF, "int:cdecl", "DBFDeleteField", "handle", $hDBF, "int", $iField)
	Return $aRet[0]
EndFunc   ;==>_DBF_DeleteField

Func _DBF_CloneEmpty($hDBF, $sNewFilename)
	;creates a new empty dbf file with the fields from already opened dbf
	Local $aRet = DllCall($hDLL_DBF, "handle:cdecl", "DBFCloneEmpty", "handle", $hDBF, "str", $sNewFilename)
	Return $aRet[0]
EndFunc   ;==>_DBF_CloneEmpty

Func _DBF_ReadTuple($hDBF, $iRecord)
	;read a complete record (binary)
	Local $aRet = DllCall($hDLL_DBF, "str:cdecl", "DBFReadTuple", "handle", $hDBF, "int", $iRecord)
	Return StringToBinary($aRet[0])
EndFunc   ;==>_DBF_ReadTuple

Func _DBF_SplitTuple($hDBF, $bTuple)
	Local $iFieldCount = _DBF_GetFieldCount($hDBF)
	Local $aTemp, $aValues[$iFieldCount], $iOffset = 2 ; tuple always starts with extra byte (0x20) !?

	For $i = 0 To $iFieldCount - 1
		$aTemp = _DBF_GetFieldInfo($hDBF, $i)
		$aValues[$i] = StringStripWS(BinaryToString(BinaryMid($bTuple, $iOffset, $aTemp[2])), 1)
		$iOffset += $aTemp[2]
	Next
	Return $aValues
EndFunc   ;==>_DBF_SplitTuple

Func _DBF_DBFToArray($hDBF)
	Local $x = _DBF_GetFieldCount($hDBF)
	Local $y = _DBF_GetRecordCount($hDBF)
	Local $aRet[$y + 1][$x]
	Local $aFieldInfo

	For $i = 0 To $x - 1
		$aFieldInfo = _DBF_GetFieldInfo($hDBF, $i)
		$aRet[0][$i] = $aFieldInfo[1]
		For $j = 1 To $y
			If _DBF_IsAttributeNULL($hDBF, $j - 1, $i) Then
				$aRet[$j][$i] = $DBF_StringNULL
			Else
				Switch $aFieldInfo[0]
					Case $DBF_FTString
						$aRet[$j][$i] = _DBF_ReadStringAttribute($hDBF, $j - 1, $i)
					Case $DBF_FTInteger
						$aRet[$j][$i] = _DBF_ReadIntegerAttribute($hDBF, $j - 1, $i)
					Case $DBF_FTDouble
						$aRet[$j][$i] = _DBF_ReadDoubleAttribute($hDBF, $j - 1, $i)
					Case $DBF_FTLogical
						$aRet[$j][$i] = _DBF_ReadLogicalAttribute($hDBF, $j - 1, $i)
				EndSwitch
			EndIf
		Next
	Next
	Return $aRet
EndFunc   ;==>_DBF_DBFToArray

